(ns world2D.blobs
  (:use quil.core 
        world2D.globals world2D.util world2D.vec2D))

(def blob-size 30)
  
(defn draw-blob [b]
  (let [[x y] (:position b)]
    (push-matrix)
    (translate x y)
    (rotate (:rotation b))
    ;; outer membrane
    (fill 128, 255, 0, 100)
    (ellipse 0 0 blob-size blob-size)
    ;; nucleus
    (fill 32, 64, 1, 196) 
    (ellipse 0 0 (/ blob-size 2) (/ blob-size 2))
    ;; heading, relative to 12 o'clock
    (fill 255, 255, 0, 196)
    (ellipse 0 (* blob-size -0.375) (* blob-size 0.25) (* blob-size 0.25))
    (pop-matrix)))

(defn blob-proposals
  [b]
  (assoc b :proposals
         (let [acceleration 
               (+v ;(*v 1 (apply avgv (map :velocity (:sensed b))))
                   #_(*v 3 [(dec (rand 2))
                           (dec (rand 2))])
                   [0 0]
                   ;; world center
                   ;(*v 2 (-v [400 400] (:position b)))
                   )
               ]
           {:acceleration acceleration
            :rotation :from-velocity})))

(defn blob-sensor ;; sense everything in neighborhood
  [b]
  (assoc b :sensed (:neighbors b)))
  
(defn blobs-colliding?
  [b1 b2]
  (< (length (-v (:position b1) (:position b2)))
     (/ blob-size 2)))

(defn random-blob
  []
  {:position [(rand-int screen-size) (rand-int screen-size)]
   :velocity [(* 5 (- (rand) 0.5)) (* 5 (- (rand) 0.5))]
   :rotation (* two-pi (rand))
   :neighbors []
   :draw-function draw-blob
   :proposal-function blob-proposals
   :sensor-function blob-sensor
   :collision-detection-function blobs-colliding?})

