(ns world2D.neighbors
  (:use world2D.globals world2D.util world2D.vec2D))

(def grid-steps ;; set at load time based on neighborhood-size
  (max 1 (int (/ screen-size neighborhood-size))))

(def grid-step-size 
  (max 1 (int (/ screen-size grid-steps))))

(defn wrap-grid-coordinate 
  [c]
  (if (< c 0)
    (dec grid-steps)
    (if (>= c grid-steps)
      0
      c)))
  
(defn wrap-gridxy
  [xy]
  (mapv wrap-grid-coordinate xy))

(defn xy->gridxy
  [xy]
  (wrap-gridxy (mapv (fn [c] (int (/ c grid-step-size))) xy)))

(defn strip-neighbors
  [obj]
  (-> obj
    (dissoc :neighbors)
    (dissoc :sensed)))

(defn make-world-grid
  [objs]
  (loop [grid (vec (repeat grid-steps (vec (repeat grid-steps []))))
         remaining (mapv strip-neighbors objs)]
    (if (empty? remaining)
      grid
      (recur (update-in grid 
                        (xy->gridxy (:position (first remaining)))
                        conj
                        (first remaining))
             (rest remaining)))))
  
(defn update-neighbors
  "Annotates each world object with :neighbors."
  []
  (swap! 
   world-objects
   (fn [objs]
     (let [grid (make-world-grid objs)
           window (fn [c] (mapv #(wrap-grid-coordinate (+ c %)) [-1 0 1]))]
       (vec (doall
              (mapv (fn [obj]
                      (assoc obj :neighbors
                             (doall
                               (filterv #(<= (length (mapv - (:position obj) (:position %)))
                                             neighborhood-size)
                                        (apply concat
                                               (let [[x y] (xy->gridxy (:position obj))]
                                                 (doall
                                                   (for [xs (window x)
                                                         ys (window y)]
                                                     (get-in grid (wrap-gridxy [xs ys]))))))))))
                    objs)))))))


