(ns world2D.physics
    (:use quil.core 
          world2D.globals world2D.util world2D.vec2D))

(defn generate-proposals
  []
  (swap! world-objects
         (fn [objs]
           (pmapall #((:proposal-function %) %) objs))))

(defn arbitrate-proposals
  []
  (swap! world-objects
         (fn [objs]
           (pmapall (fn [{:keys [position velocity rotation neighbors proposals] :as obj}]
                      (let [proposed-a (nrm (:acceleration proposals))
                            anti-collision-a (*v collision-resolution-force
                                                 (apply avgv 
                                                        (mapv #(-v position %)
                                                              (mapv :position
                                                                    (filterv #((:collision-detection-function obj) obj %)
                                                                             neighbors)))))
                            new-a (nrm-wrt (+v proposed-a anti-collision-a) max-acceleration)
                            proposed-v (+v velocity new-a)
                            new-v (nrm-wrt proposed-v max-velocity)
                            new-p (wrap-position (+v position new-v))
                            proposed-r (if (= (:rotation proposals) :from-velocity)
                                         (direction->rotation new-v)
                                         (:rotation proposals))
                            new-r (wrap-rotation
                                    (cond 
                                      ;; already there
                                      (== proposed-r rotation) 
                                      rotation
                                      ;; go up
                                      (or
                                        (and (> proposed-r rotation) 
                                             (< (- proposed-r rotation)
                                                (+ (- rotation minus-two-pi)
                                                   (- two-pi proposed-r))))
                                        (and (< proposed-r rotation)
                                             (> (- rotation proposed-r)
                                                (+ (- proposed-r minus-two-pi)
                                                   (- two-pi rotation)))))
                                      (min (+ rotation max-rotational-velocity)
                                           proposed-r)
                                      ;; go down
                                      :else
                                      (max (- rotation max-rotational-velocity)
                                           proposed-r)))]
                        (-> obj
                          (assoc :velocity new-v)
                          (assoc :position new-p)
                          (assoc :rotation new-r))))
                    objs))))