(ns world2D.util
  (:use clojure.repl quil.core world2D.globals))

(defn wrap-rotation 
  "Wraps rotation angle r to -2pi <= r <= 2pi"
  [r]
  (cond (< r minus-two-pi) (- two-pi r)
        (> r two-pi) (- r four-pi)
        :else r))

(defn wrap-position [[x y]]
  [(if (< x 0) 
     (- screen-size x)
     (if (>= x screen-size)
       (- x screen-size)
       x))
   (if (< y 0) 
     (- screen-size y)
     (if (>= y screen-size)
       (- y screen-size)
       y))])

(defn sign [n] (if (neg? n) -1 1))

#_(defn wrapped-distance
   [[x1 y1][x2 y2]]
   (let [x-dist (min (Math/abs (- x1 x2)) (Math/abs (- x2 x1)))
         y-dist (min (Math/abs (- y1 y2)) (Math/abs (- y2 y1)))]
     (Math/sqrt (+ (* x-dist x-dist) (* y-dist y-dist)))))

(defn atan2-local ;; http://en.wikipedia.org/wiki/Atan2 ... quil provides atan2 also... the same?
  [y x]
  (cond (> x 0) (Math/atan (/ y x))
        (and (>= y 0) (< x 0)) (+ (Math/atan (/ y x)) Math/PI)
        (and (< y 0) (< x 0)) (- (Math/atan (/ y x)) Math/PI)
        (and (> y 0) (= x 0)) (/ Math/PI 2)
        (and (< y 0) (= x 0)) (- (/ Math/PI 2))
        :else 0))

(defn direction->rotation
  [[x y]]
  ;;http://stackoverflow.com/questions/2276855/xna-2d-vector-angles-whats-the-correct-way-to-calculate
  (atan2-local x (- y)))

(defn pmapall
  "Like pmap but: 1) coll should be finite, 2) the returned sequence
   will not be lazy, 3) calls to f may occur in any order, to maximize
   multicore processor utilization, and 4) takes only one coll so far."
  [f coll]
  (if single-thread-mode
    (doall (map f coll))
    (let [agents (map #(agent % :error-handler 
                              (fn [agnt except] (clojure.repl/pst except 1000) (System/exit 0))) 
                      coll)]
      (dorun (map #(send % f) agents))
      (apply await agents)
      (doall (map deref agents)))))