(ns world2D.core
  ;; Lee Spector, lspector@hampshire.edu, 20140409
  (:use quil.core 
        world2D.globals world2D.util world2D.vec2D world2D.physics
        world2D.neighbors world2D.sensors world2D.draw world2D.blobs)
  (:gen-class))

(defn setup []
  (smooth)
  (no-stroke)
  ;; create initial objects
  (dotimes [_ 200] (swap! world-objects conj (random-blob)))
  (reset! iteration 0))
   
(use 'clojure.pprint)

(defn print-stats []
  (println "World objects:" (count @world-objects))
  (println "Avg neighbors" (float (/ (reduce + (map count (map :neighbors @world-objects)))
                                     (count @world-objects))))
  (when (zero? (mod @iteration 100)) 
    (println)
    (pprint (first @world-objects)))
  )

(defn draw
  []
  (swap! iteration inc)
  (update-neighbors) 
  (run-sensors)
  (generate-proposals) ;; should access only :sensed, not :neighbors (still needed for arbitration)
  (arbitrate-proposals)
  (draw-world-objects)
  ;(print-stats)
  )

#_(defn mouse-moved []
   (let [x (mouse-x)  y (mouse-y)]
     (reset! (state :mouse-position) [x y])))

(defn -main [& args]
  (sketch
    :title "World2D"
    :setup setup
    :draw draw
    :size [screen-size screen-size]
    ;:mouse-moved mouse-moved
    ;:on-close #(System/exit 0)
    ))

;(-main)

;(def foo
;{:proposals {:acceleration (0 0), :rotation :from-velocity}, 
; :neighbors [{:proposals {:acceleration (0 0), :rotation :from-velocity}, 
;              :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, 
;              :position [85.32519031064326 408.95153880639305], 
;              :velocity (2.325190310643259 -0.048461193606980646), 
;              :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, 
;              :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, 
;              :sensed [{:position [83 409], 
;                        :velocity [2.325190310643259 -0.048461193606980646], 
;                        :rotation 5.015054852860356, 
;                        :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, 
;                        :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, 
;                        :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, 
;                        :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} 
;                       {:position [61 492], :velocity [0.7878906834729926 -2.2260335007177634], 
;                        :rotation 0.47057547131308375, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [41 489], :velocity [-0.655591773600841 2.1111451641073087], :rotation 6.146238034353739, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [96 435], :velocity [1.2040710039842888 -1.0239213483565606], :rotation 5.406895844875482, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [11 473], :velocity [1.4048797567693723 2.4853245944391826], :rotation 3.2989559607848387, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [85 502], :velocity [2.0736595025835562 1.371935667862595], :rotation 1.0113912245014314, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [173 373], :velocity [2.2985085401328598 -1.6519229695310922], :rotation 2.2090360522949313, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [105 394], :velocity [0.6163891466727223 -0.34252682703671433], :rotation 0.5631131952377605, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [114 403], :velocity [2.3659822965010413 -0.6990327898957077], :rotation 0.592343554498483, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>}], :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>, :rotation 4.965054852860356} {:proposals {:acceleration (0 0), :rotation :from-velocity}, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :position [61.787890683472995 489.7739664992822], :velocity (0.7878906834729926 -2.2260335007177634), :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensed [{:position [83 409], :velocity [2.325190310643259 -0.048461193606980646], :rotation 5.015054852860356, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [61 492], :velocity [0.7878906834729926 -2.2260335007177634], :rotation 0.47057547131308375, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [41 489], :velocity [-0.655591773600841 2.1111451641073087], :rotation 6.146238034353739, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [96 435], :velocity [1.2040710039842888 -1.0239213483565606], :rotation 5.406895844875482, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [11 473], :velocity [1.4048797567693723 2.4853245944391826], :rotation 3.2989559607848387, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [21 547], :velocity [-1.1664406534843552 -0.2555353904757579], :rotation 0.7880495114546382, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [85 502], :velocity [2.0736595025835562 1.371935667862595], :rotation 1.0113912245014314, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [61 530], :velocity [0.6019356787693408 -0.8457419554525736], :rotation 2.4930077154206054, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [41 544], :velocity [-1.8113003325477757 -1.287226477148734], :rotation 1.0635099854184513, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [55 550], :velocity [0.9241127332992877 -0.8932706332939938], :rotation 3.5503114726800904, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [160 496], :velocity [0.8838951438793624 0.17207820835123566], :rotation 4.154011012390593, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [115 516], :velocity [1.8640700787631244 -0.2932187001377218], :rotation 0.5248125929344334, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [152 524], :velocity [-1.162570497665552 1.4942313054040186], :rotation 5.052338594786834, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>}], :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>, :rotation 0.42057547131308376} {:proposals {:acceleration (0 0), :rotation :from-velocity}, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :position [40.34440822639916 491.1111451641073], :velocity (-0.655591773600841 2.1111451641073087), :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensed [{:position [83 409], :velocity [2.325190310643259 -0.048461193606980646], :rotation 5.015054852860356, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [61 492], :velocity [0.7878906834729926 -2.2260335007177634], :rotation 0.47057547131308375, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [41 489], :velocity [-0.655591773600841 2.1111451641073087], :rotation 6.146238034353739, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [96 435], :velocity [1.2040710039842888 -1.0239213483565606], :rotation 5.406895844875482, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [11 473], :velocity [1.4048797567693723 2.4853245944391826], :rotation 3.2989559607848387, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [21 547], :velocity [-1.1664406534843552 -0.2555353904757579], :rotation 0.7880495114546382, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [85 502], :velocity [2.0736595025835562 1.371935667862595], :rotation 1.0113912245014314, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [61 530], :velocity [0.6019356787693408 -0.8457419554525736], :rotation 2.4930077154206054, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [41 544], :velocity [-1.8113003325477757 -1.287226477148734], :rotation 1.0635099854184513, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [55 550], :velocity [0.9241127332992877 -0.8932706332939938], :rotation 3.5503114726800904, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [115 516], :velocity [1.8640700787631244 -0.2932187001377218], :rotation 0.5248125929344334, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>}], :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>, :rotation -2.840495805211529} {:proposals {:acceleration (0 0), :rotation :from-velocity}, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :position [97.20407100398428 433.97607865164343], :velocity (1.2040710039842888 -1.0239213483565606), :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensed [{:position [83 409], :velocity [2.325190310643259 -0.048461193606980646], :rotation 5.015054852860356, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [61 492], :velocity [0.7878906834729926 -2.2260335007177634], :rotation 0.47057547131308375, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [41 489], :velocity [-0.655591773600841 2.1111451641073087], :rotation 6.146238034353739, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [96 435], :velocity [1.2040710039842888 -1.0239213483565606], :rotation 5.406895844875482, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [11 473], :velocity [1.4048797567693723 2.4853245944391826], :rotation 3.2989559607848387, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [85 502], :velocity [2.0736595025835562 1.371935667862595], :rotation 1.0113912245014314, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [173 373], :velocity [2.2985085401328598 -1.6519229695310922], :rotation 2.2090360522949313, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [105 394], :velocity [0.6163891466727223 -0.34252682703671433], :rotation 0.5631131952377605, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [114 403], :velocity [2.3659822965010413 -0.6990327898957077], :rotation 0.592343554498483, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [186 461], :velocity [-0.1422411233225246 -0.3105737433244965], :rotation 3.900245253579204, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [160 496], :velocity [0.8838951438793624 0.17207820835123566], :rotation 4.154011012390593, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [115 516], :velocity [1.8640700787631244 -0.2932187001377218], :rotation 0.5248125929344334, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>}], :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>, :rotation 5.356895844875482} {:proposals {:acceleration (0 0), :rotation :from-velocity}, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :position [12.404879756769372 475.4853245944392], :velocity (1.4048797567693723 2.4853245944391826), :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensed [{:position [83 409], :velocity [2.325190310643259 -0.048461193606980646], :rotation 5.015054852860356, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [61 492], :velocity [0.7878906834729926 -2.2260335007177634], :rotation 0.47057547131308375, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [41 489], :velocity [-0.655591773600841 2.1111451641073087], :rotation 6.146238034353739, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [96 435], :velocity [1.2040710039842888 -1.0239213483565606], :rotation 5.406895844875482, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [11 473], :velocity [1.4048797567693723 2.4853245944391826], :rotation 3.2989559607848387, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [21 547], :velocity [-1.1664406534843552 -0.2555353904757579], :rotation 0.7880495114546382, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [85 502], :velocity [2.0736595025835562 1.371935667862595], :rotation 1.0113912245014314, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [61 530], :velocity [0.6019356787693408 -0.8457419554525736], :rotation 2.4930077154206054, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [41 544], :velocity [-1.8113003325477757 -1.287226477148734], :rotation 1.0635099854184513, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [55 550], :velocity [0.9241127332992877 -0.8932706332939938], :rotation 3.5503114726800904, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>}], :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>, :rotation 3.248955960784839} {:proposals {:acceleration (0 0), :rotation :from-velocity}, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :position [87.07365950258355 503.3719356678626], :velocity (2.0736595025835562 1.371935667862595), :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensed [{:position [83 409], :velocity [2.325190310643259 -0.048461193606980646], :rotation 5.015054852860356, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [61 492], :velocity [0.7878906834729926 -2.2260335007177634], :rotation 0.47057547131308375, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [41 489], :velocity [-0.655591773600841 2.1111451641073087], :rotation 6.146238034353739, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [96 435], :velocity [1.2040710039842888 -1.0239213483565606], :rotation 5.406895844875482, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [11 473], :velocity [1.4048797567693723 2.4853245944391826], :rotation 3.2989559607848387, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [21 547], :velocity [-1.1664406534843552 -0.2555353904757579], :rotation 0.7880495114546382, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [85 502], :velocity [2.0736595025835562 1.371935667862595], :rotation 1.0113912245014314, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [61 530], :velocity [0.6019356787693408 -0.8457419554525736], :rotation 2.4930077154206054, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [41 544], :velocity [-1.8113003325477757 -1.287226477148734], :rotation 1.0635099854184513, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [55 550], :velocity [0.9241127332992877 -0.8932706332939938], :rotation 3.5503114726800904, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [160 496], :velocity [0.8838951438793624 0.17207820835123566], :rotation 4.154011012390593, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [155 570], :velocity [1.129993675753651 -0.8918711231364335], :rotation 1.399166182759315, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [115 516], :velocity [1.8640700787631244 -0.2932187001377218], :rotation 0.5248125929344334, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [152 524], :velocity [-1.162570497665552 1.4942313054040186], :rotation 5.052338594786834, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>}], :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>, :rotation 1.0613912245014314} {:proposals {:acceleration (0 0), :rotation :from-velocity}, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :position [175.29850854013287 371.3480770304689], :velocity (2.2985085401328598 -1.6519229695310922), :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensed [{:position [83 409], :velocity [2.325190310643259 -0.048461193606980646], :rotation 5.015054852860356, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [96 435], :velocity [1.2040710039842888 -1.0239213483565606], :rotation 5.406895844875482, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [175 286], :velocity [-0.265785752755589 -1.4181084848789949], :rotation 5.485436792465061, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [154 298], :velocity [-1.400122667269631 2.019740813766499], :rotation 6.271648328826186, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [123 287], :velocity [-2.0580211344796053 0.6528995923998265], :rotation 5.136498144243249, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [173 373], :velocity [2.2985085401328598 -1.6519229695310922], :rotation 2.2090360522949313, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [105 394], :velocity [0.6163891466727223 -0.34252682703671433], :rotation 0.5631131952377605, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [169 319], :velocity [2.434483914192998 -1.5356500883056552], :rotation 6.018063248911226, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [114 403], :velocity [2.3659822965010413 -0.6990327898957077], :rotation 0.592343554498483, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [186 461], :velocity [-0.1422411233225246 -0.3105737433244965], :rotation 3.900245253579204, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [253 389], :velocity [-2.2843382946988604 -0.49420280896936564], :rotation 4.999494113968604, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [216 369], :velocity [-0.9371586730624171 -0.07550092221713434], :rotation 3.625812467422369, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [218 454], :velocity [1.7053656177870429 0.5340908176089326], :rotation 1.1822999439739859, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [242 408], :velocity [-1.5237400877293865 -1.947410044956256], :rotation 1.4540703480928137, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>}], :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>, :rotation 2.1590360522949315} {:proposals {:acceleration (0 0), :rotation :from-velocity}, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :position [104.90928236548618 392.95036639177675], :velocity (-0.0907176345138252 -1.0496336082232618), :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensed [{:position [87 304], :velocity [-0.901410007856146 -0.002096380858969127], :rotation 4.355026522885678, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [83 409], :velocity [2.325190310643259 -0.048461193606980646], :rotation 5.015054852860356, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [96 435], :velocity [1.2040710039842888 -1.0239213483565606], :rotation 5.406895844875482, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [173 373], :velocity [2.2985085401328598 -1.6519229695310922], :rotation 2.2090360522949313, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [105 394], :velocity [0.6163891466727223 -0.34252682703671433], :rotation 0.5631131952377605, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [169 319], :velocity [2.434483914192998 -1.5356500883056552], :rotation 6.018063248911226, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [114 403], :velocity [2.3659822965010413 -0.6990327898957077], :rotation 0.592343554498483, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>}], :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>, :rotation 0.5131131952377604} {:proposals {:acceleration (0 0), :rotation :from-velocity}, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :position [117.07308907768758 403.0080739912908], :velocity (3.0730890776875888 0.008073991290839744), :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensed [{:position [83 409], :velocity [2.325190310643259 -0.048461193606980646], :rotation 5.015054852860356, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [96 435], :velocity [1.2040710039842888 -1.0239213483565606], :rotation 5.406895844875482, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [173 373], :velocity [2.2985085401328598 -1.6519229695310922], :rotation 2.2090360522949313, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [105 394], :velocity [0.6163891466727223 -0.34252682703671433], :rotation 0.5631131952377605, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [114 403], :velocity [2.3659822965010413 -0.6990327898957077], :rotation 0.592343554498483, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [186 461], :velocity [-0.1422411233225246 -0.3105737433244965], :rotation 3.900245253579204, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>}], :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>, :rotation 0.642343554498483}], :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :position [87.65038062128653 408.9030776127861], :velocity (2.325190310643259 -0.048461193606980646), :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensed [{:proposals {:acceleration (0 0), :rotation :from-velocity}, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :position [85.32519031064326 408.95153880639305], :velocity (2.325190310643259 -0.048461193606980646), :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensed [{:position [83 409], :velocity [2.325190310643259 -0.048461193606980646], :rotation 5.015054852860356, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [61 492], :velocity [0.7878906834729926 -2.2260335007177634], :rotation 0.47057547131308375, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [41 489], :velocity [-0.655591773600841 2.1111451641073087], :rotation 6.146238034353739, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [96 435], :velocity [1.2040710039842888 -1.0239213483565606], :rotation 5.406895844875482, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [11 473], :velocity [1.4048797567693723 2.4853245944391826], :rotation 3.2989559607848387, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [85 502], :velocity [2.0736595025835562 1.371935667862595], :rotation 1.0113912245014314, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [173 373], :velocity [2.2985085401328598 -1.6519229695310922], :rotation 2.2090360522949313, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [105 394], :velocity [0.6163891466727223 -0.34252682703671433], :rotation 0.5631131952377605, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [114 403], :velocity [2.3659822965010413 -0.6990327898957077], :rotation 0.592343554498483, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>}], :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>, :rotation 4.965054852860356} {:proposals {:acceleration (0 0), :rotation :from-velocity}, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :position [61.787890683472995 489.7739664992822], :velocity (0.7878906834729926 -2.2260335007177634), :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensed [{:position [83 409], :velocity [2.325190310643259 -0.048461193606980646], :rotation 5.015054852860356, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [61 492], :velocity [0.7878906834729926 -2.2260335007177634], :rotation 0.47057547131308375, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [41 489], :velocity [-0.655591773600841 2.1111451641073087], :rotation 6.146238034353739, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [96 435], :velocity [1.2040710039842888 -1.0239213483565606], :rotation 5.406895844875482, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [11 473], :velocity [1.4048797567693723 2.4853245944391826], :rotation 3.2989559607848387, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [21 547], :velocity [-1.1664406534843552 -0.2555353904757579], :rotation 0.7880495114546382, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [85 502], :velocity [2.0736595025835562 1.371935667862595], :rotation 1.0113912245014314, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [61 530], :velocity [0.6019356787693408 -0.8457419554525736], :rotation 2.4930077154206054, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [41 544], :velocity [-1.8113003325477757 -1.287226477148734], :rotation 1.0635099854184513, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [55 550], :velocity [0.9241127332992877 -0.8932706332939938], :rotation 3.5503114726800904, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [160 496], :velocity [0.8838951438793624 0.17207820835123566], :rotation 4.154011012390593, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [115 516], :velocity [1.8640700787631244 -0.2932187001377218], :rotation 0.5248125929344334, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [152 524], :velocity [-1.162570497665552 1.4942313054040186], :rotation 5.052338594786834, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>}], :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>, :rotation 0.42057547131308376} {:proposals {:acceleration (0 0), :rotation :from-velocity}, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :position [40.34440822639916 491.1111451641073], :velocity (-0.655591773600841 2.1111451641073087), :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensed [{:position [83 409], :velocity [2.325190310643259 -0.048461193606980646], :rotation 5.015054852860356, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [61 492], :velocity [0.7878906834729926 -2.2260335007177634], :rotation 0.47057547131308375, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [41 489], :velocity [-0.655591773600841 2.1111451641073087], :rotation 6.146238034353739, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [96 435], :velocity [1.2040710039842888 -1.0239213483565606], :rotation 5.406895844875482, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [11 473], :velocity [1.4048797567693723 2.4853245944391826], :rotation 3.2989559607848387, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [21 547], :velocity [-1.1664406534843552 -0.2555353904757579], :rotation 0.7880495114546382, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [85 502], :velocity [2.0736595025835562 1.371935667862595], :rotation 1.0113912245014314, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [61 530], :velocity [0.6019356787693408 -0.8457419554525736], :rotation 2.4930077154206054, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [41 544], :velocity [-1.8113003325477757 -1.287226477148734], :rotation 1.0635099854184513, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [55 550], :velocity [0.9241127332992877 -0.8932706332939938], :rotation 3.5503114726800904, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [115 516], :velocity [1.8640700787631244 -0.2932187001377218], :rotation 0.5248125929344334, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>}], :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>, :rotation -2.840495805211529} {:proposals {:acceleration (0 0), :rotation :from-velocity}, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :position [97.20407100398428 433.97607865164343], :velocity (1.2040710039842888 -1.0239213483565606), :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensed [{:position [83 409], :velocity [2.325190310643259 -0.048461193606980646], :rotation 5.015054852860356, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [61 492], :velocity [0.7878906834729926 -2.2260335007177634], :rotation 0.47057547131308375, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [41 489], :velocity [-0.655591773600841 2.1111451641073087], :rotation 6.146238034353739, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [96 435], :velocity [1.2040710039842888 -1.0239213483565606], :rotation 5.406895844875482, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [11 473], :velocity [1.4048797567693723 2.4853245944391826], :rotation 3.2989559607848387, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [85 502], :velocity [2.0736595025835562 1.371935667862595], :rotation 1.0113912245014314, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [173 373], :velocity [2.2985085401328598 -1.6519229695310922], :rotation 2.2090360522949313, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [105 394], :velocity [0.6163891466727223 -0.34252682703671433], :rotation 0.5631131952377605, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [114 403], :velocity [2.3659822965010413 -0.6990327898957077], :rotation 0.592343554498483, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [186 461], :velocity [-0.1422411233225246 -0.3105737433244965], :rotation 3.900245253579204, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [160 496], :velocity [0.8838951438793624 0.17207820835123566], :rotation 4.154011012390593, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [115 516], :velocity [1.8640700787631244 -0.2932187001377218], :rotation 0.5248125929344334, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>}], :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>, :rotation 5.356895844875482} {:proposals {:acceleration (0 0), :rotation :from-velocity}, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :position [12.404879756769372 475.4853245944392], :velocity (1.4048797567693723 2.4853245944391826), :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensed [{:position [83 409], :velocity [2.325190310643259 -0.048461193606980646], :rotation 5.015054852860356, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [61 492], :velocity [0.7878906834729926 -2.2260335007177634], :rotation 0.47057547131308375, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [41 489], :velocity [-0.655591773600841 2.1111451641073087], :rotation 6.146238034353739, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [96 435], :velocity [1.2040710039842888 -1.0239213483565606], :rotation 5.406895844875482, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [11 473], :velocity [1.4048797567693723 2.4853245944391826], :rotation 3.2989559607848387, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [21 547], :velocity [-1.1664406534843552 -0.2555353904757579], :rotation 0.7880495114546382, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [85 502], :velocity [2.0736595025835562 1.371935667862595], :rotation 1.0113912245014314, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [61 530], :velocity [0.6019356787693408 -0.8457419554525736], :rotation 2.4930077154206054, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [41 544], :velocity [-1.8113003325477757 -1.287226477148734], :rotation 1.0635099854184513, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [55 550], :velocity [0.9241127332992877 -0.8932706332939938], :rotation 3.5503114726800904, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>}], :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>, :rotation 3.248955960784839} {:proposals {:acceleration (0 0), :rotation :from-velocity}, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :position [87.07365950258355 503.3719356678626], :velocity (2.0736595025835562 1.371935667862595), :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensed [{:position [83 409], :velocity [2.325190310643259 -0.048461193606980646], :rotation 5.015054852860356, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [61 492], :velocity [0.7878906834729926 -2.2260335007177634], :rotation 0.47057547131308375, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [41 489], :velocity [-0.655591773600841 2.1111451641073087], :rotation 6.146238034353739, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [96 435], :velocity [1.2040710039842888 -1.0239213483565606], :rotation 5.406895844875482, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [11 473], :velocity [1.4048797567693723 2.4853245944391826], :rotation 3.2989559607848387, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [21 547], :velocity [-1.1664406534843552 -0.2555353904757579], :rotation 0.7880495114546382, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [85 502], :velocity [2.0736595025835562 1.371935667862595], :rotation 1.0113912245014314, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [61 530], :velocity [0.6019356787693408 -0.8457419554525736], :rotation 2.4930077154206054, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [41 544], :velocity [-1.8113003325477757 -1.287226477148734], :rotation 1.0635099854184513, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [55 550], :velocity [0.9241127332992877 -0.8932706332939938], :rotation 3.5503114726800904, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [160 496], :velocity [0.8838951438793624 0.17207820835123566], :rotation 4.154011012390593, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [155 570], :velocity [1.129993675753651 -0.8918711231364335], :rotation 1.399166182759315, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [115 516], :velocity [1.8640700787631244 -0.2932187001377218], :rotation 0.5248125929344334, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [152 524], :velocity [-1.162570497665552 1.4942313054040186], :rotation 5.052338594786834, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>}], :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>, :rotation 1.0613912245014314} {:proposals {:acceleration (0 0), :rotation :from-velocity}, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :position [175.29850854013287 371.3480770304689], :velocity (2.2985085401328598 -1.6519229695310922), :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensed [{:position [83 409], :velocity [2.325190310643259 -0.048461193606980646], :rotation 5.015054852860356, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [96 435], :velocity [1.2040710039842888 -1.0239213483565606], :rotation 5.406895844875482, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [175 286], :velocity [-0.265785752755589 -1.4181084848789949], :rotation 5.485436792465061, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [154 298], :velocity [-1.400122667269631 2.019740813766499], :rotation 6.271648328826186, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [123 287], :velocity [-2.0580211344796053 0.6528995923998265], :rotation 5.136498144243249, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [173 373], :velocity [2.2985085401328598 -1.6519229695310922], :rotation 2.2090360522949313, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [105 394], :velocity [0.6163891466727223 -0.34252682703671433], :rotation 0.5631131952377605, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [169 319], :velocity [2.434483914192998 -1.5356500883056552], :rotation 6.018063248911226, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [114 403], :velocity [2.3659822965010413 -0.6990327898957077], :rotation 0.592343554498483, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [186 461], :velocity [-0.1422411233225246 -0.3105737433244965], :rotation 3.900245253579204, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [253 389], :velocity [-2.2843382946988604 -0.49420280896936564], :rotation 4.999494113968604, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [216 369], :velocity [-0.9371586730624171 -0.07550092221713434], :rotation 3.625812467422369, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [218 454], :velocity [1.7053656177870429 0.5340908176089326], :rotation 1.1822999439739859, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [242 408], :velocity [-1.5237400877293865 -1.947410044956256], :rotation 1.4540703480928137, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>}], :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>, :rotation 2.1590360522949315} {:proposals {:acceleration (0 0), :rotation :from-velocity}, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :position [104.90928236548618 392.95036639177675], :velocity (-0.0907176345138252 -1.0496336082232618), :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensed [{:position [87 304], :velocity [-0.901410007856146 -0.002096380858969127], :rotation 4.355026522885678, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [83 409], :velocity [2.325190310643259 -0.048461193606980646], :rotation 5.015054852860356, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [96 435], :velocity [1.2040710039842888 -1.0239213483565606], :rotation 5.406895844875482, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [173 373], :velocity [2.2985085401328598 -1.6519229695310922], :rotation 2.2090360522949313, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [105 394], :velocity [0.6163891466727223 -0.34252682703671433], :rotation 0.5631131952377605, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [169 319], :velocity [2.434483914192998 -1.5356500883056552], :rotation 6.018063248911226, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [114 403], :velocity [2.3659822965010413 -0.6990327898957077], :rotation 0.592343554498483, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>}], :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>, :rotation 0.5131131952377604} {:proposals {:acceleration (0 0), :rotation :from-velocity}, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :position [117.07308907768758 403.0080739912908], :velocity (3.0730890776875888 0.008073991290839744), :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensed [{:position [83 409], :velocity [2.325190310643259 -0.048461193606980646], :rotation 5.015054852860356, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [96 435], :velocity [1.2040710039842888 -1.0239213483565606], :rotation 5.406895844875482, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [173 373], :velocity [2.2985085401328598 -1.6519229695310922], :rotation 2.2090360522949313, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [105 394], :velocity [0.6163891466727223 -0.34252682703671433], :rotation 0.5631131952377605, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [114 403], :velocity [2.3659822965010413 -0.6990327898957077], :rotation 0.592343554498483, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>} {:position [186 461], :velocity [-0.1422411233225246 -0.3105737433244965], :rotation 3.900245253579204, :draw-function #<blobs$draw_blob world2D.blobs$draw_blob@1d646723>, :proposal-function #<blobs$blob_proposals world2D.blobs$blob_proposals@5602c43c>, :sensor-function #<blobs$blob_sensor world2D.blobs$blob_sensor@6e88a13e>, :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>}], :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>, :rotation 0.642343554498483}], :collision-detection-function #<blobs$blobs_colliding_QMARK_ world2D.blobs$blobs_colliding_QMARK_@3b85ead9>, :rotation 4.915054852860356}
;)